% program wyner's common information of a tuple of Gaussian random variables
%   Function for the computation of the wci in bits and in nats
%
%   Jan H. van Schuppen (Author)
%   Gouden Leeuw 143
%   1103 KB Amsterdam
%   Netherlands
%   jan.h.van.schuppen@xs4all.nl
%   2017:01:04  date file started
%   2017:01:19  date of this version of the file
%
%
function [p11, p12, p13, p21, p22, p23, S1, S2, QQ, QQr, wcibits, wcinats] = wcigrvsfunc(p,p1,p2,p12min,Q,threshold1,threshold2)

%
% Test output of the input variables
%
% E = eig(Q)
%
%
%  Step 1. Compute the partition of the output variance
%
Q11 = Q(1:p1,1:p1);
Q22 = Q(p1+1:p,p1+1:p);
Q12 = Q(1:p1,p1+1:p);
%
%  Test output
%
% Q11
% Q12
% Q22
%
%  Step 2. SVD of the two variances
%
[W11,D1,W12] = svd(Q11);
[W21,D2,W22] = svd(Q22);
%
% test output
% D1
% D2
% Q11
% QQQ11a= W11 * D1 * W11';
%
% 
%  Step 3. SVD of the correlation matrix
%
D11 = eye(p1);
D22 = eye(p2);
for i = 1:p1
    D11(i,i) = 1/sqrt(D1(i,i));
end
for i = 1:p2
    D22(i,i) = 1/sqrt(D2(i,i));
end
%
% Test output
% D11
% D22
%
W11tr = W11';
W22tr = W22';
Q3 = D11 * W11tr * Q12 * W22 * D22;
%
% Test output
% Q3
%
[W3,QQ12,W4] = svd(Q3);
U3 = W3;
U4 = W4;
%
%  Test output
%
% QQ11 = D11 * W11tr * Q11 * W11 * D11;
% QQ11
% QQ22 = D22 * W22tr * Q22 * W22 * D22;
% QQ22
% QQ12
% U3
% U4
%
%  Step 4.  Computation of the variance matrix of the canonical variable form
%
QQ = eye(p);
QQ(1:p1,p1+1:p1+p2) = QQ12(1:p1,1:p2);
QQ12tr = QQ12'; 
QQ(p1+1:p,1:p1) = QQ12tr(1:p2,1:p1);
%
% Test output
% EQQ = eig(QQ)
%
%  Step 5.  Computation of the indices of the canonical variable form
%
p11 = 0;
p13 = 0;
for i = 1:p12min
   if (QQ12(i,i) > threshold1)
      p11 = p11 +1;
   else
     if (QQ12(i,i) < threshold2)
        p13 = p13 + 1;
     end
   end
end
if (p1-p2 > 0)
   p13 = p13 + p1-p2;
end
diff1 = p1 - p11 - p13;
if (diff1 < 0)
   p13 = p13 + diff1;
end
diff2 = p2 - p1 + p13;
if (diff2 < 0)
	p13 = p13 - diff2;
end
p12 = p1 - p11 - p13;
p21 = p11;
p22 = p12;
p23 = p2 - p21 - p22;
%
% test output
%
% p11, p21,
% p12, p22,
% p13, p23,
% QQ
%
% Step 6.  Compute the modification of the QQ matrix.
%
QQr = QQ;
if p11 > 0
   for i=1:p11
     QQr(i,p1+i) = 1;
     QQr(p1+i,i) = 1;
   end
end
if p13 > 0
   for i=1:p13
     QQr(p11+p12+i:p1+p11+p12+i) = 0;
   end
end
if p23 > 0
   for i=1:p23
     QQr(p1+p21+p22+i,i) = 0;
   end
end
%
% Test output
% QQr
%
% 
%  Step 7. Computation of the transformation matrices to cv-form
%
S1 = U3' * D11 * W11';
S2 = U4' * D22 * W22';
%
% Test output
% QQQQ11 = S1 * Q11 * S1';
% QQQQ22 = S2 * Q22 * S2'
% QQQQ12 = S1 * Q12 * S2';
% QQQQ11
% QQQQ22
% QQQQ12
%
%  Step 8. Computation of the common information
%
sumbits = 0;
sumnats = 0;
for i = 1:p12
    j = p11 + i;
    r6 = [1+QQr(j,p1+j)]/[1-QQr(j,p1+j)];
    sumbits = sumbits + log2(r6);
    sumnats = sumnats + log(r6);
end
wcibits = sumbits;
wcinats = sumnats;
%
%
%  end of function
%


